"use client";
import {
  Box,
  Button,
  ButtonGroup,
  Grid,
  Input,
  Slider,
  Stack,
  styled,
  TextField,
  ToggleButton,
  ToggleButtonGroup,
  Typography,
} from "@mui/material";
import React from "react";
import { StripBusOutput, StripBusOutputEvent } from "./StripBusOutput";
import { EventCounter } from "@/utils/EventCounter";
import { stopPropagation } from "@/utils/StopPropagation";
import { getComponentState } from "@/utils/GetComponentState";

export interface StripProps {
  physicalBuses: number;
  virtualBuses: number;
  values?: Partial<StripState>;
  width: number;
  name: string;
  onChange: (event: StripEvent) => any;
}

export type StateDefaultValueException = "";

export interface StripState {
  gain: number;
  outputBuses: boolean[];
  muted: boolean;
}

export type StripEvent = StripBusOutputChanged | StripMuted | StripGainChanged;

export interface StripBusOutputChanged extends StripBusOutputEvent {
  type: "StripBusOutputChanged";
}

export interface StripGainChanged {
  type: "StripGainChanged";
  gain: number;
}
export interface StripMuted {
  type: "StripMuted";
  muted: boolean;
}

export class Strip extends React.Component<StripProps, StripState> {
  private defaultValues: Omit<StripState, StateDefaultValueException>;
  private eventCounter = new EventCounter<"onSliderResetDefaults">();
  constructor(props: StripProps) {
    super(props);
    const { values: initialValues } = props;
    this.defaultValues = {
      gain: 0,
      outputBuses: [...Array(props.physicalBuses + props.virtualBuses)].map(
        (_v, k) => false
      ),
      muted: false,
    };

    this.state = getComponentState(initialValues, this.defaultValues);
    this.eventCounter.on({
      callback: this.onResetDefaults.bind(this),
      count: 2,
      name: "onSliderResetDefaults",
    });
  }

  onSliderClick(event: React.MouseEvent) {
    stopPropagation(event);
    this.eventCounter.emit("onSliderResetDefaults");
  }
  onKeyDown(event: React.KeyboardEvent) {
    console.log(event);

    if (event.key === "ArrowLeft" || event.key === "ArrowRight") {
      event.preventDefault();
    }
  }
  _setGain(gain: number) {
    this.setState({
      gain,
    });
    this.props.onChange({
      type: "StripGainChanged",
      gain,
    });
  }
  onResetDefaults() {
    this._setGain(this.defaultValues.gain);
  }

  onGainSliderChange(event: Event) {
    stopPropagation(event);
    const multiplier = (event as unknown as React.KeyboardEvent).shiftKey
      ? 2
      : 1;
    const changedGain =
      parseFloat((event.target as HTMLInputElement).value) * multiplier;
    this._setGain(changedGain);
  }
  onBusChange(event: StripBusOutputEvent) {
    const buses = this.state.outputBuses;
    buses[
      event.isVirtual ? this.props.physicalBuses + event.busId : event.busId
    ] = event.enabled;
    this.setState({
      outputBuses: buses,
    });
    this.props.onChange({
      type: "StripBusOutputChanged",
      busId: event.busId,
      isVirtual: event.isVirtual,
      enabled: event.enabled,
    });
  }
  onStripWheel(event: React.WheelEvent) {
    console.log(event);
  }
  onMuteToggle(event: React.MouseEvent) {
    stopPropagation(event);
    this.setState(
      (state) => ({ muted: !state.muted }),
      () => {
        this.props.onChange({ type: "StripMuted", muted: this.state.muted });
      }
    );
  }
  render() {
    return (
      <>
        <Stack
          direction={"row"}
          sx={{
            ...(this.props.width <= 1027 && { marginBlockEnd: "10px" }),
          }}
        >
          <Stack alignItems={"center"} style={{ maxWidth: "70px" }}>
            <Slider
              orientation="vertical"
              valueLabelDisplay={this.props.width > 600 ? "off" : "auto"}
              value={this.state.gain}
              color={
                this.state.gain > 0 && this.state.gain < 12
                  ? "warning"
                  : this.state.gain <= 0
                  ? "primary"
                  : "error"
              }
              min={-60}
              max={12}
              step={0.1}
              slotProps={{
                root: {
                  onClick: (e) => this.onSliderClick(e),
                },
              }}
              onChange={(e) => this.onGainSliderChange(e)}
            />
          </Stack>
          <Stack>
            <ButtonGroup orientation="vertical" variant="text" color="inherit" disableElevation={true}>
              {[
                ...Array(this.props.physicalBuses + this.props.virtualBuses),
              ].map((_v, i) => (
                <StripBusOutput
                  key={`${i < this.props.physicalBuses ? "a" : "b"}${
                    i < this.props.physicalBuses
                      ? i
                      : i - this.props.physicalBuses
                  }`}
                  id={
                    i < this.props.physicalBuses
                      ? i
                      : i - this.props.physicalBuses
                  }
                  isVirtual={i < this.props.physicalBuses ? false : true}
                  onChange={(e) => this.onBusChange(e)}
                  default={{ enabled: this.state.outputBuses[i] }}
                />
              ))}
              <Button
                onClick={(e) => this.onMuteToggle(e)}
                variant={this.state.muted ? "contained" : "outlined"}
                color={"error"}
                size="small"
              >
                Mute
              </Button>
            </ButtonGroup>
          </Stack>
        </Stack>
      </>
    );
  }
}
