"use client";
import React, { createRef, useEffect, useMemo, useRef, useState } from "react";
import { Strip, StripEvent, StripProps, StripState } from "./component/Strip";
import {
  Backdrop,
  Button,
  CircularProgress,
  Container,
  createTheme,
  CssBaseline,
  Divider,
  Fab,
  Slide,
  Snackbar,
  Stack,
  ThemeProvider,
  Typography,
  useMediaQuery,
} from "@mui/material";
import { Bus, BusEvent, BusProps, BusState } from "./component/Bus";
import Grid from "@mui/material/Unstable_Grid2";
import { EventCounter } from "@/utils/EventCounter";
import { BusesList } from "./component/BusesList";
import { isItMobile } from "@/utils/DetectMobile";
import FullscreenIcon from "@mui/icons-material/Fullscreen";
import FullscreenExit from "@mui/icons-material/FullscreenExit";

function random(min: number, max: number, floor: boolean = true) {
  const value = Math.random() * (max - min + 1) + min;
  return floor ? Math.floor(value) : value;
}

interface Values {
  strips: { gain: number; muteState: boolean; outputBuses: boolean[] }[];
  buses: { gain: number; muteState: boolean }[];
}

export default function Home() {
  const theme = useMemo(
    () =>
      createTheme({
        palette: {
          mode: "dark",
          background: { default: "#000" },
          primary: { main: "#9d9d9d" },
          secondary: { main: "#262626" },
          error: { main: "#cb0801", A700: "#fff" },
          warning: { main: "#cb7c01" },
        },
      }),
    []
  );
  const isSnackBarHidden = useMemo(() => {
    if (typeof localStorage === "undefined") return false;
    return localStorage.getItem("snackBarHidden") === "true";
  }, []);
  const [snackBarVisible, setSnackBarVisibility] = useState(false);
  const [values, setValues] = useState<{
    buses: BusProps["values"][];
    strips: StripProps["values"][];
  }>();
  const [width, setWidth] = useState(1000);
  const [isLoaded, setIsLoaded] = useState(false);
  const [isFullscreen, setIsFullscreen] = useState(false);
  let [apiUrl, setApiUrl] = useState<string>();

  const isItMobileDevice = useMemo(
    () =>
      typeof window !== "undefined" ? isItMobile(navigator.userAgent) : "",
    []
  );
  function sendApi(
    path: string,
    method: RequestInit["method"] = "GET",
    body?: any,
    fetchOptions?: RequestInit
  ) {
    const url = new URL(path, apiUrl);
    return fetch(url.toString(), {
      method,
      body: body !== undefined ? JSON.stringify(body) : undefined,
      headers: (() =>
        body !== undefined && body !== null
          ? { "Content-Type": "application/json" }
          : ({} as RequestInit["headers"]))(),
      ...fetchOptions,
    })
      .then((r) => r.json())
      .catch((e) => console.error("Error:", e));
  }
  function onEvent(event: StripEvent | BusEvent, id: number) {
    console.log(event);
    switch (event.type) {
      case "StripMuted": {
        sendApi(`/strip/${id}/mute`, "PATCH", { muteState: event.muted });
        break;
      }
      case "StripGainChanged": {
        sendApi(`/strip/${id}/gain`, "PATCH", { gain: event.gain });
        break;
      }
      case "BusMuted": {
        sendApi(`/bus/${id}/mute`, "PATCH", { muteState: event.muted });
        break;
      }
      case "BusGainChanged": {
        sendApi(`/bus/${id}/gain`, "PATCH", { gain: event.gain });
        break;
      }
      case "StripBusOutputChanged":
        setValues((e) => {
          if (e === undefined) return;
          e.strips[id]!.outputBuses![
            event.isVirtual ? event.busId + buses.physical : event.busId
          ] = event.enabled;
          sendApi(`/strip/${id}/outputs`, "PATCH", {
            outputs: e.strips[id]!.outputBuses,
          });
          return e;
        });

        break;
      case "BusSelectToggle":
        break;
    }
  }
  const toggleFullscreen = () => {
    new Promise((r) => {
      if (document.fullscreenElement === null)
        r(document.documentElement.requestFullscreen());
      else r(document.exitFullscreen());
    }).then(() => setIsFullscreen(document.fullscreenElement !== null));
  };
  useEffect(() => {
    const _apiUrl = new URL(document.URL);
    _apiUrl.port = "3001";
    setApiUrl(_apiUrl.toString());
    apiUrl = _apiUrl.toString();

    sendApi("/values")
      .then((v: Values) => {
        setValues({
          buses: v.buses.map((v) => ({
            gain: v.gain,
            muted: v.muteState,
          })),
          strips: v.strips.map((v) => ({
            gain: v.gain,
            muted: v.muteState,
            outputBuses: v.outputBuses,
          })),
        });
        setIsLoaded(true);
      })
      .catch((e) => console.error(e));
    setSnackBarVisibility(!isSnackBarHidden);
    setWidth(window.innerWidth);
    window.onresize = () => {
      setWidth(window.innerWidth);
    };
  }, []);
  const strips = { virtual: 3, physical: 5 };
  const buses = { virtual: 3, physical: 5 };
  const amountOfStrips = strips.physical + strips.virtual;
  const amountOfBuses = buses.physical + buses.virtual;
  const virtualInputNames = useMemo(() => ["VAIO", "AUX", "VAIO3"], []);

  const snackBarActionButton = (
    <>
      <Button
        onClick={() => {
          localStorage.setItem("snackBarHidden", "true");
          setSnackBarVisibility(false);
        }}
        color="secondary"
      >
        OK
      </Button>
    </>
  );
  return (
    <ThemeProvider theme={theme}>
      <Container
        id="container"
        maxWidth={"xl"}
        style={{ height: "100%", paddingInline: "10px" }}
      >
        <CssBaseline />
        {isLoaded ? (
          <div>
            <Typography variant="h5">Inputs</Typography>
            <Grid
              container
              // spacing={width ? 0 : 1}
              sx={{ paddingBlockStart: "15px" }}
              display={"flex"}
              alignItems={"center"}
              justifyContent={"space-evenly"}
              columns={{ xs: 8, sm: 12, md: 12, lg: 14 }}
            >
              {[...Array(amountOfStrips)].map((_v, stripId) => {
                const isPhysical = stripId < strips.physical;
                const name = `${isPhysical ? "A" : "B"}${
                  isPhysical ? stripId + 1 : stripId + 1 - strips.physical
                }`;
                return (
                  <React.Fragment key={name}>
                    <Grid
                      lgOffset={stripId === strips.physical ? 2 : 0}
                      mdOffset={
                        stripId === strips.physical
                          ? width > 1098
                            ? 0.8
                            : 0
                          : 0
                      }
                      sx={{ minWidth: "fit-content" }}
                    >
                      <Strip
                        onChange={(ev) => onEvent(ev, stripId)}
                        physicalBuses={buses.physical}
                        values={values?.strips[stripId]}
                        virtualBuses={buses.virtual}
                        width={width}
                        name={
                          isPhysical
                            ? `#${stripId + 1}`
                            : virtualInputNames[stripId - strips.physical]
                        }
                      />
                    </Grid>
                  </React.Fragment>
                );
              })}
            </Grid>
            <Typography variant="h5">Outputs</Typography>
            <BusesList
              onChange={onEvent}
              stackProps={{ sx: { paddingBlockStart: "10px" } }}
              width={width}
              values={[...Array(amountOfBuses)].map(
                (_v, busId) => values?.buses[busId]
              )}
              physical={buses.physical}
              virtual={buses.virtual}
            />
          </div>
        ) : (
          <Backdrop
            sx={(theme) => ({ color: "#fff", zIndex: theme.zIndex.drawer + 1 })}
            open={!isLoaded}
          >
            <CircularProgress color="inherit" />
          </Backdrop>
        )}
      </Container>
      <Snackbar
        anchorOrigin={{ horizontal: "center", vertical: "top" }}
        TransitionComponent={Slide}
        transitionDuration={250}
        open={snackBarVisible}
        message={
          <>
            <Typography fontWeight={600}>Tips:</Typography>
            <Typography component={"span"}>
              <Typography variant={"button"} fontWeight={500}>
                Double {isItMobileDevice ? "tap" : "click"}
              </Typography>{" "}
              on any gain slider to reset it&apos;s value to 0 dB
              <br />
            </Typography>
            {/* <Typography component={"span"}>
              <Typography variant={"button"} fontWeight={500}>
                Triple {isItMobileDevice ? "tap" : "click"}
              </Typography>{" "}
              on empty space for toggle fullscreen
            </Typography> */}
          </>
        }
        action={snackBarActionButton}
      />
      <Fab
        size="medium"
        sx={{
          position: "fixed",
          bottom: 15,
          right: 15,
          zIndex: theme.zIndex.drawer + 2,
        }}
        color="secondary"
        onClick={() => toggleFullscreen()}
      >
        {isFullscreen ? <FullscreenExit /> : <FullscreenIcon />}
      </Fab>
    </ThemeProvider>
  );
}
